@@ -40,6 +40,19 @@ impl Helm {
         }
     }
 
+    pub(crate) fn skip(self) -> Self {
+        match self {
+            Self::Do {
+                namespace,
+                default_storage_class,
+            } => Self::Skip {
+                namespace,
+                default_storage_class,
+            },
+            other => other,
+        }
+    }
+
     pub(crate) fn default_storage_class(&self) -> Option<&str> {
         match self {
             Helm::Skip {
@@ -101,7 +114,7 @@ impl Helm {
                     .arg("--version")
                     .arg(&helm.version)
                     .arg(&helm.chart);
-                for (param, value) in &helm.paramarers {
+                for (param, value) in &helm.parameters {
                     cmd.arg("--set").arg(format!("{}={}", param, value));
                 }
                 if let Some(default_storage_class) = default_storage_class {
