@@ -76,8 +76,8 @@ enum Command {
 
     #[structopt(about = "Register new cluster", aliases = &["register-cl", "rc"], display_order(10))]
     RegisterCluster {
-        #[structopt(help = "Cluster name")]
-        name: v1::ClusterName,
+        #[structopt(help = "Cluster name, defaults to current k8s context")]
+        name: Option<v1::ClusterName>,
 
         #[structopt(
             help = "List of states to for this cluster to use",
@@ -174,10 +174,24 @@ enum Command {
     },
 
     #[structopt(about = "Manually create new volume", display_order(50))]
-    CreateVolume,
+    CreateVolume {
+        #[structopt(help = "State name")]
+        state: v1::StateName,
+        #[structopt(help = "Volume name")]
+        volume: v1::VolumeName,
+        #[structopt(help = "valume size")]
+        size: u64,
+        #[structopt(help = "volume file system")]
+        fs_type: v1::VolumeFileSystem,
+    },
 
     #[structopt(about = "Manually delete existing volume", display_order(50))]
-    DeleteVolume,
+    DeleteVolume {
+        #[structopt(help = "State name")]
+        state: v1::StateName,
+        #[structopt(help = "Volume name")]
+        volume: v1::VolumeName,
+    },
 
     #[structopt(
         about = "Create new namespace",
@@ -278,6 +292,13 @@ impl Cli {
                 namespace,
                 skip_helm,
             } => {
+                let name = if let Some(name) = name.or_else(k8s::get_default_cluster_name) {
+                    name
+                } else {
+                    anyhow::bail!(
+                        "No default Kubernetes context found, need to provide cluster name"
+                    );
+                };
                 let no_default_storage_class = if no_state {
                     true
                 } else {
@@ -309,8 +330,13 @@ impl Cli {
             Command::SetAvailability => statehub.set_availability().await,
             Command::SetOwner { state, cluster } => statehub.set_owner(state, cluster).await,
             Command::UnsetOwner { state, cluster } => statehub.unset_owner(state, cluster).await,
-            Command::CreateVolume => statehub.create_volume().await,
-            Command::DeleteVolume => statehub.delete_volume().await,
+            Command::CreateVolume {
+                state,
+                volume,
+                size,
+                fs_type,
+            } => statehub.create_volume(state, volume, size, fs_type).await,
+            Command::DeleteVolume { state, volume } => statehub.delete_volume(state, volume).await,
             Command::CreateNamespace { namespace } => statehub.create_namespace(namespace).await,
             Command::SaveClusterToken { namespace, token } => {
                 statehub.save_cluster_token(namespace, token).await
@@ -356,7 +382,7 @@ impl StateHub {
         &self,
         name: v1::StateName,
         owner: Option<v1::ClusterName>,
-        locations: v1::Locations,
+        locations: v1::CreateStateLocationsDto,
     ) -> anyhow::Result<()> {
         let state = v1::CreateStateDto {
             name,
@@ -381,7 +407,7 @@ impl StateHub {
     }
 
     pub(crate) async fn list_clusters(&self) -> anyhow::Result<()> {
-        self.api.list_clusters().await.handle_output(self.json)
+        self.api.get_all_clusters().await.handle_output(self.json)
     }
 
     async fn register_cluster(
@@ -391,37 +417,44 @@ impl StateHub {
         helm: k8s::Helm,
         claim_unowned_states: bool,
     ) -> anyhow::Result<()> {
-        if !k8s::helm_is_found() {
-            anyhow::bail!(
-                "helm is not detected, please make sure helm is installed and is in the PATH"
-            );
-        }
+        let helm = if k8s::helm_is_found() {
+            helm
+        } else {
+            log::warn!("helm is not detected. Will show helm command instead of executing them");
+            helm.skip()
+        };
 
+        let locations = k8s::collect_node_locations().await?;
+        let provider = k8s::get_cluster_provider(&cluster);
+
+        let cluster = self
+            .api
+            .register_cluster(&cluster, provider, &locations)
+            .await?;
+
+        log::info!("Registering {}", cluster.show());
         if let Some(ref states) = states {
-            let locations = k8s::collect_node_locations().await?;
             self.adjust_all_states(states, &locations).await?;
         } else {
             self.verbosely("## Skip adding this cluster to any state");
         }
 
-        let output = self.api.register_cluster(&cluster).await?;
-
         k8s::validate_namespace(helm.namespace()).await?;
 
-        self.setup_cluster_token_helper(&output, &helm).await?;
-        self.setup_configmap_helper(&output, &helm).await?;
+        self.setup_cluster_token_helper(&cluster, &helm).await?;
+        self.setup_configmap_helper(&cluster, &helm).await?;
 
-        helm.execute(&output, self.verbose).await?;
+        helm.execute(&cluster, self.verbose).await?;
 
         if claim_unowned_states {
             if let Some(states) = states {
                 for state in states {
-                    self.api.set_owner(state, &cluster).await?;
+                    self.api.set_owner(state, &cluster.name).await?;
                 }
             }
         }
 
-        output.handle_output(self.json)
+        cluster.handle_output(self.json)
     }
 
     async fn unregister_cluster(&self, name: v1::ClusterName, force: bool) -> anyhow::Result<()> {
@@ -441,21 +474,55 @@ impl StateHub {
     }
 
     async fn add_location(&self, state: v1::StateName, location: Location) -> anyhow::Result<()> {
-        self.add_location_helper(&state, &location).await
+        let state = self.api.get_state(&state).await?;
+        if state.is_available_in(&location) {
+            log::info!("{} is already available in {}", state, location);
+        } else {
+            self.add_location_helper(&state, &location).await?;
+        }
+        Ok(())
     }
 
     async fn remove_location(self, state: v1::StateName, location: Location) -> anyhow::Result<()> {
-        self.remove_location_helper(&state, &location).await
+        let state = self.api.get_state(&state).await?;
+        if state.is_available_in(&location) {
+            self.remove_location_helper(&state, &location).await?;
+        } else {
+            log::info!("{} is not availabe in {}", state, location);
+        }
+        Ok(())
     }
 
-    pub(crate) async fn create_volume(self) -> anyhow::Result<()> {
+    async fn create_volume(
+        self,
+        state_name: v1::StateName,
+        volume_name: v1::VolumeName,
+        size: u64,
+        fs: v1::VolumeFileSystem,
+    ) -> anyhow::Result<()> {
+        let volume = v1::CreateVolumeDto {
+            name: volume_name.to_string(),
+            size_gi: size,
+            fs_type: fs.to_string(),
+        };
+
+        self.api
+            .create_volume(state_name, volume)
+            .await
+            .handle_output(self.json)
         // Ok(Output::<String>::todo()).handle_output(self.json)
-        anyhow::bail!(self.show(Output::<String>::todo()))
+        //anyhow::bail!(self.show(Output::<String>::todo()))
     }
 
-    pub(crate) async fn delete_volume(self) -> anyhow::Result<()> {
-        // Ok(Output::<String>::todo()).handle_output(self.json)
-        anyhow::bail!(self.show(Output::<String>::todo()))
+    pub(crate) async fn delete_volume(
+        self,
+        state: v1::StateName,
+        volume: v1::VolumeName,
+    ) -> anyhow::Result<()> {
+        self.api
+            .delete_volume(state, volume)
+            .await
+            .handle_output(self.json)
     }
 
     pub(crate) async fn set_availability(self) -> anyhow::Result<()> {
@@ -479,10 +546,16 @@ impl StateHub {
         state: v1::StateName,
         cluster: v1::ClusterName,
     ) -> anyhow::Result<()> {
-        self.api
-            .unset_owner(state, cluster)
-            .await
-            .handle_output(self.json)
+        let state = self.api.get_state(&state).await?;
+
+        if state.owner == Some(cluster) {
+            self.api
+                .unset_owner(&state.name)
+                .await
+                .handle_output(self.json)
+        } else {
+            anyhow::bail!("Permission denied, you are not theowner of this state.")
+        }
     }
 
     pub(crate) fn show<T>(&self, output: Output<T>) -> String
