@@ -7,7 +7,7 @@ use std::collections::HashMap;
 
 use chrono::{DateTime, Utc};
 use serde::{Deserialize, Serialize};
-use serde_with::{DeserializeFromStr, SerializeDisplay};
+use serde_with::{skip_serializing_none, DeserializeFromStr, SerializeDisplay};
 use uuid::Uuid;
 
 mod impls;
@@ -22,11 +22,12 @@ pub struct State {
     pub created: DateTime<Utc>,
     pub modified: DateTime<Utc>,
     pub storage_class: Option<StorageClass>,
-    pub locations: Locations,
+    #[serde(default)]
+    pub locations: StateLocations,
     pub owner: Option<ClusterName>,
     pub provisioning_status: ProvisioningStatus,
     pub allowed_clusters: Option<Vec<ClusterName>>,
-    pub condition: Option<Condition>,
+    pub condition: Condition,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
@@ -34,7 +35,7 @@ pub struct State {
 pub struct CreateStateDto {
     pub name: StateName,
     pub storage_class: Option<StorageClass>,
-    pub locations: Locations,
+    pub locations: CreateStateLocationsDto,
     pub owner: Option<ClusterName>,
     pub allowed_clusters: Option<Vec<ClusterName>>,
 }
@@ -55,9 +56,20 @@ pub enum Condition {
     Red,
 }
 
+#[derive(Clone, Copy, Debug, Serialize, Deserialize, PartialEq, PartialOrd, Hash)]
+#[serde(rename_all = "lowercase")]
+pub enum Provider {
+    Eks,
+    Aks,
+    Kops,
+    Generic,
+}
+
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct CreateClusterDto {
     pub name: ClusterName,
+    pub provider: Provider,
+    pub locations: ClusterLocations,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
@@ -66,7 +78,8 @@ pub struct Cluster {
     pub name: ClusterName,
     pub created: DateTime<Utc>,
     pub modified: DateTime<Utc>,
-    pub locations: Locations,
+    #[serde(default)]
+    pub locations: ClusterLocations,
     pub helm: Vec<Helm>,
 }
 
@@ -80,7 +93,7 @@ pub struct Helm {
     pub repo: String,
     pub chart: String,
     pub version: String,
-    pub paramarers: HashMap<String, String>,
+    pub parameters: HashMap<String, String>,
 }
 
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
@@ -90,25 +103,75 @@ pub struct StateName(pub String);
 pub struct ClusterName(pub String);
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
-pub struct CreateStateLocationAws {
+pub struct CreateStateLocationsDto {
+    pub aws: Vec<CreateStateLocationAwsDto>,
+    pub azure: Vec<CreateStateLocationAzureDto>,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct CreateStateLocationAwsDto {
     pub region: AwsRegion,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
-pub struct CreateStateLocationAzure {
+pub struct CreateStateLocationAzureDto {
     pub region: AzureRegion,
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
-pub struct StateLocationAws {
-    pub region: AwsRegion,
-    pub status: StateLocationStatus,
+#[derive(Clone, Debug, Default, Serialize, Deserialize)]
+pub struct StateLocations {
+    #[serde(default)]
+    pub aws: Vec<StateLocationAws>,
+    #[serde(default)]
+    pub azure: Vec<StateLocationAzure>,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
 pub struct StateLocationAzure {
     pub region: AzureRegion,
     pub status: StateLocationStatus,
+    pub volumes: Vec<VolumeLocation>,
+    pub private_link_service: Option<PrivateLinkServiceAzure>,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct StateLocationAws {
+    pub region: AwsRegion,
+    pub status: StateLocationStatus,
+    pub volumes: Vec<VolumeLocation>,
+    pub private_link_service: Option<PrivateLinkServiceAws>,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct PrivateLinkServiceAws {
+    pub id: String,
+    pub name: String,
+}
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct PrivateLinkServiceAzure {
+    pub id: String,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct VolumeLocation {
+    pub status: LocationVolumeStatus,
+    pub progress: Option<StateLocationVolumeProgress>,
+    pub name: String,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct LocationVolumeStatus {
+    pub value: StateLocationStatus,
+    pub msg: Option<String>,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct StateLocationVolumeProgress {
+    pub bytes_syncronized: u64,
+    pub bytes_total: u64,
 }
 
 #[derive(Clone, Copy, Debug, Serialize, Deserialize, PartialEq, PartialOrd, Hash)]
@@ -117,10 +180,12 @@ pub enum StateLocationStatus {
     Ok,
     Provisioning,
     Recovering,
+    Deleting,
     Error,
 }
 
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
 pub struct StorageClass {
     pub mount_options: Option<String>,
     pub volume_binding_mode: VolumeBindingMode,
@@ -133,9 +198,21 @@ pub enum VolumeBindingMode {
 }
 
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
-pub struct Locations {
-    aws: Vec<AwsRegion>,
-    azure: Vec<AzureRegion>,
+pub struct ClusterLocations {
+    pub aws: Vec<ClusterLocationAws>,
+    pub azure: Vec<ClusterLocationAzure>,
+}
+#[skip_serializing_none]
+#[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct ClusterLocationAws {
+    pub region: AwsRegion,
+    pub account_principal: Option<String>,
+}
+
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct ClusterLocationAzure {
+    pub region: AzureRegion,
 }
 
 #[derive(
@@ -183,6 +260,9 @@ pub enum GcpRegion {
     Antarctica,
 }
 
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+pub struct VolumeName(pub String);
+
 #[derive(Clone, Debug, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Volume {
@@ -190,8 +270,10 @@ pub struct Volume {
     pub name: String,
     pub size_gi: u64,
     pub fs_type: String,
-    pub active_location: Locations,
-    pub status: VolumeStatus,
+    pub active_location: Option<String>,
+    pub locations: Vec<VolumeLocation>,
+    pub created: DateTime<Utc>,
+    pub modified: DateTime<Utc>,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
@@ -202,6 +284,20 @@ pub struct CreateVolumeDto {
     pub fs_type: String,
 }
 
+#[derive(
+    Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash, SerializeDisplay, DeserializeFromStr,
+)]
+pub enum VolumeFileSystem {
+    Ext,
+    Ext2,
+    Ext3,
+    Ext4,
+    Jfs,
+    Swap,
+    Fat,
+    Fat32,
+}
+
 #[derive(
     Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash, SerializeDisplay, DeserializeFromStr,
 )]
