@@ -13,10 +13,15 @@ use super::*;
 mod aws;
 mod azure;
 mod gcp;
+mod volume;
 
 impl fmt::Display for State {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("State").field("name", &self.name).finish()
+        f.debug_struct("State")
+            .field("name", &self.name.0)
+            .field("aws", &self.locations.aws)
+            .field("azure", &self.locations.azure)
+            .finish()
     }
 }
 
@@ -96,14 +101,41 @@ impl Default for Condition {
     }
 }
 
-impl From<Vec<Location>> for Locations {
-    fn from(locations: Vec<Location>) -> Self {
+impl From<AwsRegion> for CreateStateLocationAwsDto {
+    fn from(region: AwsRegion) -> Self {
+        Self { region }
+    }
+}
+
+impl From<AzureRegion> for CreateStateLocationAzureDto {
+    fn from(region: AzureRegion) -> Self {
+        Self { region }
+    }
+}
+
+impl From<AwsRegion> for ClusterLocationAws {
+    fn from(region: AwsRegion) -> Self {
+        Self {
+            region,
+            account_principal: None,
+        }
+    }
+}
+
+impl From<AzureRegion> for ClusterLocationAzure {
+    fn from(region: AzureRegion) -> Self {
+        Self { region }
+    }
+}
+
+impl From<&[Location]> for ClusterLocations {
+    fn from(locations: &[Location]) -> Self {
         let mut aws = vec![];
         let mut azure = vec![];
         for location in locations {
             match location {
-                Location::Aws(region) => aws.push(region),
-                Location::Azure(region) => azure.push(region),
+                Location::Aws(region) => aws.push((*region).into()),
+                Location::Azure(region) => azure.push((*region).into()),
                 // Location::Gcp(region) => gcp.push(region),
             }
         }
@@ -111,11 +143,26 @@ impl From<Vec<Location>> for Locations {
     }
 }
 
-impl Locations {
+impl From<Vec<Location>> for CreateStateLocationsDto {
+    fn from(locations: Vec<Location>) -> Self {
+        let mut aws = vec![];
+        let mut azure = vec![];
+        for location in locations {
+            match location {
+                Location::Aws(region) => aws.push(region.into()),
+                Location::Azure(region) => azure.push(region.into()),
+                // Location::Gcp(region) => gcp.push(region),
+            }
+        }
+        Self { aws, azure }
+    }
+}
+
+impl StateLocations {
     pub(crate) fn contains(&self, location: &Location) -> bool {
         match location {
-            Location::Aws(region) => self.aws.contains(region),
-            Location::Azure(region) => self.azure.contains(region),
+            Location::Aws(region) => self.aws.iter().any(|aws| aws.region == *region),
+            Location::Azure(region) => self.azure.iter().any(|azure| azure.region == *region),
         }
     }
 }
